@@ -325,7 +325,11 @@ robj *tryObjectEncoding(robj *o) {
      * Note that we also avoid using shared integers when maxmemory is used
      * because every object needs to have a private LRU field for the LRU
      * algorithm to work well. */
-    if (server.maxmemory == 0 && value >= 0 && value < REDIS_SHARED_INTEGERS) {
+    if ((server.maxmemory == 0 ||
+         (server.maxmemory_policy != REDIS_MAXMEMORY_VOLATILE_LRU &&
+          server.maxmemory_policy != REDIS_MAXMEMORY_ALLKEYS_LRU)) &&
+        value >= 0 && value < REDIS_SHARED_INTEGERS)
+    {
         decrRefCount(o);
         incrRefCount(shared.integers[value]);
         return shared.integers[value];
